/* This file is Copyright 2000 Level Control Systems.  See the included LICENSE.txt file for details. */  

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include "netutil/NetworkUtilityFunctions.h"
#include "support/PortableMacros.h"

int Connect(const char * hostName, int port, const char * debugTitle, bool errorsOnly)
{
   if ((debugTitle)&&(errorsOnly == false)) 
   {
      LogTime(LOG_INFO, "%s: Connecting to [%s:%i]: ", debugTitle, hostName, port);
      fflush(stdout);
   }
   struct hostent * he = gethostbyname(hostName);
   if (he)
   {
      int s = socket(AF_INET, SOCK_STREAM, 0);
      if (s >= 0)
      {
         struct sockaddr_in saAddr;
         memset(&saAddr, 0, sizeof(saAddr));
         saAddr.sin_family = AF_INET;
         saAddr.sin_port   = htons(port);
         memcpy(&saAddr.sin_addr.s_addr, he->h_addr, he->h_length);

         if (connect(s, (struct sockaddr *) &saAddr, sizeof(saAddr)) >= 0)
         {
            if ((debugTitle)&&(errorsOnly == false)) Log(LOG_INFO, "Connected!\n", debugTitle, hostName, port);
            return s;
         }
         else if (debugTitle) 
         {
            if (errorsOnly) Log(LOG_INFO, "%s: connect() to [%s:%i] failed!\n", debugTitle, hostName, port);
                       else Log(LOG_INFO, "Connection failed!\n");
         }
         closesocket(s);
      }
      else if (debugTitle) 
      {
         if (errorsOnly) Log(LOG_INFO, "%s: socket() failed!\n", debugTitle);
                    else Log(LOG_INFO, "socket() failed!\n");
      }
   }
   else if (debugTitle) 
   {
      if (errorsOnly) Log(LOG_INFO,"%s: Couldn't lookup hostname [%s]\n", debugTitle, hostName);
                 else Log(LOG_INFO,"hostname lookup failed!\n");
   }
   return -1;
}

int CreateAcceptSocket(int listenPort)
{
   return -1;  // TODO: implement this
}


bool IsLessThan(const struct timeval & t1, const struct timeval & t2)
{
   return (t1.tv_sec == t2.tv_sec) ? (t1.tv_usec < t2.tv_usec) : (t1.tv_sec < t2.tv_sec);
}

void AddTimeVal(struct timeval & addToThis, const struct timeval & addThis)
{
   addToThis.tv_sec  += addThis.tv_sec;
   addToThis.tv_usec += addThis.tv_usec;
   if (addToThis.tv_usec > 1000000L)
   {
      addToThis.tv_sec += addToThis.tv_usec / 1000000L;
      addToThis.tv_usec = addToThis.tv_usec % 1000000L;
   }
}                         

void SubtractTimeVal(struct timeval & subtractFromThis, const struct timeval & subtractThis)
{
   subtractFromThis.tv_sec  -= subtractThis.tv_sec;
   subtractFromThis.tv_usec -= subtractThis.tv_usec;
   if (subtractFromThis.tv_usec < 0L)
   {
      subtractFromThis.tv_sec += (subtractFromThis.tv_usec / 1000000L)-1;
      while(subtractFromThis.tv_usec < 0L) subtractFromThis.tv_usec += 1000000L;
   }
}


void Inet_NtoA(uint32 addr, char * buf)
{
   sprintf(buf, "%li.%li.%li.%li", (addr>>24)&0xFF, (addr>>16)&0xFF, (addr>>8)&0xFF, (addr>>0)&0xFF);
}

