/* This file is Copyright 2000 Level Control Systems.  See the included LICENSE.txt file for details. */

//! author="Jeremy Friesner"

#ifndef NETWORK_UTILITY_FUNCTIONS_H
#define NETWORK_UTILITY_FUNCTIONS_H

#include <sys/time.h>

#include "support/PortableTypeDefs.h"

//:Convenience method for connecting with TCP to a given hostName/port.
// Returns the sockfd if the connection is successful, -1 otherwise.
// If (debugTitle) is set, debug output will be enabled and debug output
// will be prefaced by this string.
// If (debugOutputOnErrorsOnly) is true, debug output will be printed
// only if an error condition occurs.
int Connect(const char * hostName, int port, const char * debugTitle = NULL, bool debugOutputOnErrorsOnly = true);

//:Returns a socket fd that is ready to accept TCP connections on the given port, or -1 on failure.
// (you may pass this socket to select() or accept()...).  If (listenPort) is zero, the system will
// choose a port to listen on.
// Note:  NOT IMPLEMENTED YET!
int CreateAcceptSocket(int listenPort);

// Returns true iff (t1 < t2)
bool IsLessThan(const struct timeval & t1, const struct timeval & t2);

// Adds (addThis) to (addToThis)
void AddTimeVal(struct timeval & addToThis, const struct timeval & addThis);  

// Subtracts (subtractThis) from (subtractFromThis);
void SubtractTimeVal(struct timeval & subtractFromThis, const struct timeval & subtractThis);

// Returns a string representation of the IP address (address).  Useful on OS's where inet_ntoa
// isn't available.  (outBuf) should be at least 13 bytes long
void Inet_NtoA(uint32 address, char * outBuf);

#endif
