/* This file is Copyright 2000 Level Control Systems.  See the included LICENSE.txt file for details. */  

#include <new.h>
#include <stdio.h>
#include "support/PortableMacros.h"
#include "regex/StringMatcher.h"

#include <string.h>

StringMatcher::StringMatcher() : _regExpValid(false)
{
    /* empty */
} 

StringMatcher :: StringMatcher(const char * str) : _regExpValid(false)
{
    SetPattern(str);
}

StringMatcher::~StringMatcher()
{
    if (_regExpValid) regfree(&_regExp);
}

bool StringMatcher::SetPattern(const char * str, bool isSimple) 
{
    char * freeMe = NULL;  // in case we are forced to allocate a string

    if (isSimple)
    {
       char staticBuf[200];  // for the common case (short strings), avoid the mem-alloc
       char * pattern = staticBuf;  // optimistic default
       const char * ptr = str;
       char * patptr;

       int duals = 0;
       while (*ptr != '\0') {
           switch ( *ptr ) {
           case ',':  // "\|"
           case '*':  // ".*"
           case '\\': // "\\"
           case '.':  // "\."
               duals++;
               break;
           default:
               break;
           }
           ptr++;
       }

       int total = strlen( str ) + duals + 6; // real regexp size + init/exit
       if (total+1 > sizeof(staticBuf)) freeMe = pattern = new (nothrow) char [ total + 1 ];
       if (pattern == NULL) 
       {
          WARN_OUT_OF_MEMORY;
          return false;
       }
       strcpy(pattern, "^\\(");

       ptr = str;
       patptr = pattern + 3;

       while ( *ptr != '\0' ) {
           switch ( *ptr ) {
   //      translate
           case ',': *patptr++ = '\\';
                     *patptr = '|';
                     break;
           case '*': *patptr++ = '.';
                     *patptr = '*';
                     break;
           case '?': *patptr = '.';
                     break;
   //      just quote, except when quoting the above specials
           case '\\': if ( ptr[1] == ',' ) {
                          ptr++;
                          *patptr = ',';
                      } else if ( ptr[1] == '*' ) {
                          ptr++;
                          *patptr++ = '\\';
                          *patptr = '*';
                      } else if ( ptr[1] == '?' ) {
                          ptr++;
                          *patptr++ = '\\';
                          *patptr = '?';
                      }
                      else {
                          *patptr = '\\';
                      }
                      break;
           case '.':
                     *patptr++ = '\\';
                     *patptr = *ptr;
                     break;
   //      copy verbatim
           default:  *patptr = *ptr;
                     break;
           }
           patptr++;
           ptr++;
       }
       *patptr = '\0';
       strcat( pattern, "\\)$" );

       // fprintf( stdout, "OUTPUT: pattern became '%s'.\n", pattern );
       str = pattern;
   }

   // Free the old regular expression, if any
   if (_regExpValid)    
   {
      regfree(&_regExp);
      _regExpValid = false;
   }

   // And compile the new one
   _regExpValid = (regcomp(&_regExp, str, 0) == 0);
   if (freeMe) delete [] freeMe;
   return _regExpValid;
}

bool StringMatcher::Match(const char * const str)
{
    if (_regExpValid == false) return false;
    int regExpStat = regexec(&_regExp, str, 0, NULL, 0);
    return (regExpStat != REG_NOMATCH);
} 
