/* This file is Copyright 2000 Level Control Systems.  See the included LICENSE.txt file for details. */

//! author="Jeremy Friesner"

/******************************************************************************
/
/	File:		PortableByteOrder.h
/
/	Description:	Endian correction functions
/
*******************************************************************************/

#ifndef _PORTABLEBYTEORDER_H
#define _PORTABLEBYTEORDER_H

#ifdef __BEOS__
#include <support/ByteOrder.h>  // might as well use the real thing (and avoid complaints about duplication)
#else
#include <support/PortableTypeDefs.h>
#include <endian.h>  // POSIX include, defines BYTE_ORDER as __LITTLE_ENDIAN or __BIG_ENDIAN

/*-------------------------------------------------------------*/
/*--------- Host is Little  --------------------------------------*/

#if BYTE_ORDER == __LITTLE_ENDIAN
#define B_HOST_IS_LENDIAN 1
#define B_HOST_IS_BENDIAN 0

/*--------- Host Native -> Little  --------------------*/
#define B_HOST_TO_LENDIAN_DOUBLE(arg)	(double)(arg)
#define B_HOST_TO_LENDIAN_FLOAT(arg)	(float)(arg)
#define B_HOST_TO_LENDIAN_INT64(arg)	(uint64)(arg)
#define B_HOST_TO_LENDIAN_INT32(arg)	(uint32)(arg)
#define B_HOST_TO_LENDIAN_INT16(arg)	(uint16)(arg)

/*--------- Host Native -> Big  ------------------------*/
#define B_HOST_TO_BENDIAN_DOUBLE(arg)	SWAP_DOUBLE(arg)
#define B_HOST_TO_BENDIAN_FLOAT(arg)	SWAP_FLOAT(arg)
#define B_HOST_TO_BENDIAN_INT64(arg)	SWAP_INT64(arg)
#define B_HOST_TO_BENDIAN_INT32(arg)	SWAP_INT32(arg)
#define B_HOST_TO_BENDIAN_INT16(arg)	SWAP_INT16(arg)

/*--------- Little -> Host Native ---------------------*/
#define B_LENDIAN_TO_HOST_DOUBLE(arg)	(double)(arg)
#define B_LENDIAN_TO_HOST_FLOAT(arg)	(float)(arg)
#define B_LENDIAN_TO_HOST_INT64(arg)	(uint64)(arg)
#define B_LENDIAN_TO_HOST_INT32(arg)	(uint32)(arg)
#define B_LENDIAN_TO_HOST_INT16(arg)	(uint16)(arg)

/*--------- Big -> Host Native ------------------------*/
#define B_BENDIAN_TO_HOST_DOUBLE(arg)	SWAP_DOUBLE(arg)
#define B_BENDIAN_TO_HOST_FLOAT(arg)	SWAP_FLOAT(arg)
#define B_BENDIAN_TO_HOST_INT64(arg)	SWAP_INT64(arg)
#define B_BENDIAN_TO_HOST_INT32(arg)	SWAP_INT32(arg)
#define B_BENDIAN_TO_HOST_INT16(arg)	SWAP_INT16(arg)

#else /* __LITTLE_ENDIAN */

/*-------------------------------------------------------------*/
/*--------- Host is Big  --------------------------------------*/

#define B_HOST_IS_LENDIAN 0
#define B_HOST_IS_BENDIAN 1

/*--------- Host Native -> Little  -------------------*/
#define B_HOST_TO_LENDIAN_DOUBLE(arg)	SWAP_DOUBLE(arg)
#define B_HOST_TO_LENDIAN_FLOAT(arg)	SWAP_FLOAT(arg)
#define B_HOST_TO_LENDIAN_INT64(arg)	SWAP_INT64(arg)
#define B_HOST_TO_LENDIAN_INT32(arg)	SWAP_INT32(arg)
#define B_HOST_TO_LENDIAN_INT16(arg)	SWAP_INT16(arg)

/*--------- Host Native -> Big  ------------------------*/
#define B_HOST_TO_BENDIAN_DOUBLE(arg)	(double)(arg)
#define B_HOST_TO_BENDIAN_FLOAT(arg)	(float)(arg)
#define B_HOST_TO_BENDIAN_INT64(arg)	(uint64)(arg)
#define B_HOST_TO_BENDIAN_INT32(arg)	(uint32)(arg)
#define B_HOST_TO_BENDIAN_INT16(arg)	(uint16)(arg)

/*--------- Little -> Host Native ----------------------*/
#define B_LENDIAN_TO_HOST_DOUBLE(arg)	SWAP_DOUBLE(arg)
#define B_LENDIAN_TO_HOST_FLOAT(arg)	SWAP_FLOAT(arg)
#define B_LENDIAN_TO_HOST_INT64(arg)	SWAP_INT64(arg)
#define B_LENDIAN_TO_HOST_INT32(arg)	SWAP_INT32(arg)
#define B_LENDIAN_TO_HOST_INT16(arg)	SWAP_INT16(arg)

/*--------- Big -> Host Native -------------------------*/
#define B_BENDIAN_TO_HOST_DOUBLE(arg)	(double)(arg)
#define B_BENDIAN_TO_HOST_FLOAT(arg)	(float)(arg)
#define B_BENDIAN_TO_HOST_INT64(arg)	(uint64)(arg)
#define B_BENDIAN_TO_HOST_INT32(arg)	(uint32)(arg)
#define B_BENDIAN_TO_HOST_INT16(arg)	(uint16)(arg)

#endif /* __LITTLE_ENDIAN */

// A handy little method to swap the bytes of any int type around
// I hope this is correct!  --jaf
template<class T> T swapBytes(T swapMe);
template<class T> T swapBytes(T swapMe)
{
   T retVal;
   const uint8 * readFrom = (const uint8 *) &swapMe;
   uint8 * writeTo  = ((uint8 *) &retVal)+sizeof(retVal);
   for (int i=0; i<sizeof(swapMe); i++) *(--writeTo) = *(readFrom++);
   return retVal;
}

/*-------------------------------------------------------------*/
/*--------- Just-do-it macros ---------------------------------*/
#define SWAP_DOUBLE(arg)   swapBytes((double)arg);
#define SWAP_FLOAT(arg)    swapBytes((float)arg);
#define SWAP_INT64(arg)    swapBytes((uint64)arg);
#define SWAP_INT32(arg)    swapBytes((uint32)arg);
#define SWAP_INT16(arg)    swapBytes((uint16)arg);

/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/

#endif /* !__BEOS__ */

#endif /* _PORTABLEBYTEORDER_H */

